MODULE sbccpl
   !!======================================================================
   !!                       ***  MODULE  sbccpl  ***
   !! Ocean forcing:  momentum, heat and freshwater coupled formulation
   !!=====================================================================
   !! History :  9.0   !  06-07  (R. Redler, N. Keenlyside, W. Park) 
   !!                            Original code split into flxmod & taumod
   !!            9.0   !  06-07  (G. Madec)  surface module
   !!----------------------------------------------------------------------
#if defined key_oasis3 || defined key_oasis4
   !!----------------------------------------------------------------------
   !!   'key_oasis3' or 'key_oasis4'   Coupled Ocean/Atmosphere formulation
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   namsbc_cpl   : coupled formulation namlist
   !!   sbc_cpl      : coupled formulation for the ocean surface boundary condition
   !!----------------------------------------------------------------------

   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbc_ice         ! Surface boundary condition: ice fields
#if defined key_lim3
   USE par_ice          ! ice parameters
#endif
   USE cpl_oasis3      ! OASIS3 coupling (to ECHAM5)
   USE geo2ocean
   USE restart
   USE in_out_manager  ! I/O manager
   USE iom             ! NetCDF library
   USE lib_mpp         ! distribued memory computing library
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE oce, ONLY : tn, un, vn
   USE phycst, ONLY : rt0
   USE albedo

   IMPLICIT NONE

#  include "vectopt_loop_substitute.h90"
   PRIVATE

   PUBLIC   sbc_cpl_snd       ! routine called by step.F90
   PUBLIC   sbc_cpl_rcv       ! routine called by step.F90
   
   INTEGER :: jprcv_qsroce, jprcv_qsrice, jprcv_qsrmix                                           ! Index used for qsr
   INTEGER :: jprcv_qnsoce, jprcv_qnsice, jprcv_qnsmix                                           ! Index used for qns
   INTEGER :: jprcv_rain, jprcv_snow, jprcv_tevp, jprcv_ievp, jprcv_tpre, jprcv_spre, jprcv_oemp ! Index used for water flux
   INTEGER :: jprcv_otx1, jprcv_oty1, jprcv_otz1, jprcv_otx2, jprcv_oty2, jprcv_otz2             ! Index used for wind stress on oce
   INTEGER :: jprcv_itx1, jprcv_ity1, jprcv_itz1, jprcv_itx2, jprcv_ity2, jprcv_itz2             ! Index used for wind stress on ice
   INTEGER :: jprcv_w10m                                                                         ! Index used for 10m wind
   INTEGER :: jprcv_dqnsdt                                                                       ! Index used for dqnsdt
   INTEGER :: jprcv_rnf                                                                          ! Index used for runoff
   INTEGER :: jprcv_cal                                                                          ! Index used for calving
   
   INTEGER :: jpsnd_fice                                                                         ! Index used for ice fraction 
   INTEGER :: jpsnd_toce, jpsnd_tice, jpsnd_tmix                                                 ! Index used for temperature
   INTEGER :: jpsnd_albice, jpsnd_albmix                                                         ! Index used for albedo
   INTEGER :: jpsnd_tckice, jpsnd_tcksnw                                                         ! Index used for thickness
   INTEGER :: jpsnd_ocx1, jpsnd_ocy1, jpsnd_ocz1, jpsnd_ocx2, jpsnd_ocy2, jpsnd_ocz2             ! Index used for current velocity
   INTEGER :: jpsnd_ivx1, jpsnd_ivy1, jpsnd_ivz1, jpsnd_ivx2, jpsnd_ivy2, jpsnd_ivz2             ! Index used for ice velocity
   INTEGER, DIMENSION(jpi,jpj) :: mskneg, mskpos
             ! Masks uses for negative runoff suppression

   CHARACTER(len=100) :: cn_snd_temperature, cn_snd_albedo, cn_snd_thickness,      &             ! Description of coupled mode
      cn_snd_current_1, cn_snd_current_2, cn_snd_current_3, cn_snd_current_4,      &
      cn_rcv_w10m, cn_rcv_stress_1, cn_rcv_stress_2, cn_rcv_stress_3,              &
      cn_rcv_stress_4,cn_rcv_dqnsdt, cn_rcv_qsr, cn_rcv_qns, cn_rcv_emp,           &
      cn_rcv_cal

   CHARACTER(len=100), PUBLIC :: cn_rcv_rnf
   
   CHARACTER(len=100), DIMENSION(4) :: cn_snd_current, cn_rcv_stress

   REAL(wp) :: zcumulneg, zcumulpos
             ! Temporary buffer for runoff averages
   REAL(wp), DIMENSION(jpi,jpj) :: cpl_ocean_albedo, albedo_oce_cs, albedo_oce_ov                ! Values for ocean albedo sent to atmosphere

   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2006) 
   !! $ Id: $
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

 CONTAINS
  
   SUBROUTINE sbc_cpl_init

!!! >> Arnaud pour lire freeze, alb_ice et tn_ice dans le restart oasis
     USE netcdf

     INTEGER :: ncid, varid,ierr
!!! << Arnaud
    
     CHARACTER(len=1), DIMENSION(100) :: cltmp
     INTEGER :: i

     NAMELIST/namsbc_cpl/ cn_snd_temperature, cn_snd_albedo, cn_snd_thickness,       &          
        cn_snd_current_1, cn_snd_current_2, cn_snd_current_3, cn_snd_current_4,      &
        cn_rcv_w10m, cn_rcv_stress_1, cn_rcv_stress_2, cn_rcv_stress_3,              &
        cn_rcv_stress_4, cn_rcv_dqnsdt, cn_rcv_qsr, cn_rcv_qns, cn_rcv_emp,          &
        cn_rcv_rnf, cn_rcv_cal
     

     !!---------------------------------------------------------------------
     
    
     REWIND( numnam )                    ! ... read in namlist namsbc_cpl_rcv
     READ  ( numnam, namsbc_cpl )
    
     cn_snd_current(1)=cn_snd_current_1 ; cn_snd_current(2)=cn_snd_current_2
     cn_snd_current(3)=cn_snd_current_3 ; cn_snd_current(4)=cn_snd_current_4
     cn_rcv_stress(1)=cn_rcv_stress_1   ; cn_rcv_stress(2)=cn_rcv_stress_2
     cn_rcv_stress(3)=cn_rcv_stress_3   ; cn_rcv_stress(4)=cn_rcv_stress_4
     
     !-------------------------------------
     !-------------------------------------
     ! Define the receive interface
     !-------------------------------------
     !-------------------------------------
     !
     ! Read restart of variables for coupling (needed to compute some values from the received data)


!!$   Probleme: definir comment on initialise freeze, alb_ice et tn_ice 
!!$             quand on n'a pas de restart (a nit000)
!!  >> Eric: pour le premier pas de temps du premier jour de la simulation
!!  >>       je prefere faire un truc vraiment crade
!!  >>       vu que c est vraiment la premiere et derniere occasion

!!! >> Arnaud pour lire freeze, alb_ice et tn_ice dans le restart oasis

!         CALL iom_get( numror, jpdom_local_full, 'freeze' , freeze  )

!!!     ierr = nf90_OPEN('sstoc.nc',NF90_NOWRITE,ncid)

!!!     ierr = NF90_INQ_VARID(ncid,'OIceFrac',varid)
!!!     ierr = NF90_GET_VAR(ncid,varid,freeze)

!!!     ierr = NF90_INQ_VARID(ncid,'O_AlbIce',varid)
!!!     ierr = NF90_GET_VAR(ncid,varid,alb_ice)

!!!     ierr = NF90_INQ_VARID(ncid,'O_TepIce',varid)
!!!     ierr = NF90_GET_VAR(ncid,varid,tn_ice)

!!!     ierr = NF90_close(ncid)

!!! << Arnaud

!!!!!
!!!!! +++ ERIC tu utilises tn_ice dans le calcule de Qns, c'est bien ca???
!!!!!
!     Eric : oui mais cette valeur est deja sauvee dans le restart glace
!            c est donc peut etre inutile de la sauver  aussi dans le restart ocean
!
!     IF ( TRIM(cn_rcv_qsr) == 'mixed oce-ice' ) CALL iom_get( numror, jpdom_local_full, 'alb_ice', alb_ice )
!     IF ( TRIM(cn_rcv_qns) == 'mixed oce-ice' ) CALL iom_get( numror, jpdom_local_full, 'tn_ice' , tn_ice  )

     ! default definitions of srcv
     nrcv = 0
     srcv(:)%laction = .FALSE.
     srcv(:)%clgrid = 'T'
     srcv(:)%nsgn = 1

     !-------------------------------------
     ! Qsr
     nrcv = nrcv + 1   ;   jprcv_qsroce = nrcv   ;   srcv(nrcv)%clname = 'O_QsrOce'
     nrcv = nrcv + 1   ;   jprcv_qsrice = nrcv   ;   srcv(nrcv)%clname = 'O_QsrIce'
     nrcv = nrcv + 1   ;   jprcv_qsrmix = nrcv   ;   srcv(nrcv)%clname = 'O_QsrMix'
     SELECT CASE (TRIM(cn_rcv_qsr))
     CASE( 'conservative'  )   ;   srcv( (/jprcv_qsrice, jprcv_qsrmix/) )%laction = .TRUE.
     CASE( 'oce and ice'   )   ;   srcv( (/jprcv_qsrice, jprcv_qsroce/) )%laction = .TRUE.
     CASE( 'mixed oce-ice' )   ;   srcv(                 jprcv_qsrmix   )%laction = .TRUE. 
     CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_qsr' )
     END SELECT

     !-------------------------------------
     ! Qns
     nrcv = nrcv + 1   ;   jprcv_qnsoce = nrcv   ;   srcv(nrcv)%clname = 'O_QnsOce'
     nrcv = nrcv + 1   ;   jprcv_qnsice = nrcv   ;   srcv(nrcv)%clname = 'O_QnsIce'
     nrcv = nrcv + 1   ;   jprcv_qnsmix = nrcv   ;   srcv(nrcv)%clname = 'O_QnsMix'
     SELECT CASE (TRIM(cn_rcv_qns))
     CASE( 'conservative'  )   ;   srcv( (/jprcv_qnsice, jprcv_qnsmix/) )%laction = .TRUE.
     CASE( 'oce and ice'   )   ;   srcv( (/jprcv_qnsice, jprcv_qnsoce/) )%laction = .TRUE.
     CASE( 'mixed oce-ice' )   ;   srcv(                 jprcv_qnsmix   )%laction = .TRUE. 
     CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_qns' )
     END SELECT

     !-------------------------------------
     ! emp, tprecip and sprecip
     nrcv = nrcv + 1   ;   jprcv_rain = nrcv   ;   srcv(nrcv)%clname = 'OIceRain'   ! Rain = liquid precipitation
     nrcv = nrcv + 1   ;   jprcv_snow = nrcv   ;   srcv(nrcv)%clname = 'OIceSnow'   
     nrcv = nrcv + 1   ;   jprcv_tevp = nrcv   ;   srcv(nrcv)%clname = 'OTotEvap'   ! total evaporation ( over oce + ice )
     nrcv = nrcv + 1   ;   jprcv_ievp = nrcv   ;   srcv(nrcv)%clname = 'OIceEvap'   ! evaporation iver ice (sublimation)
     nrcv = nrcv + 1   ;   jprcv_tpre = nrcv   ;   srcv(nrcv)%clname = 'OIPr-Sub'   ! Pr = liquid + solid precipitation
     nrcv = nrcv + 1   ;   jprcv_spre = nrcv   ;   srcv(nrcv)%clname = 'OISn-Sub'   ! Sub = Sublimation = Evap over ice
     nrcv = nrcv + 1   ;   jprcv_oemp = nrcv   ;   srcv(nrcv)%clname = 'OOEv-OPr'   ! 
     SELECT CASE (TRIM(cn_rcv_emp))
     CASE( 'conservative'  )   ;   srcv( (/jprcv_rain, jprcv_snow, jprcv_ievp, jprcv_tevp/) )%laction = .TRUE.
     CASE( 'oce and ice'   )   ;   srcv( (/            jprcv_tpre, jprcv_spre, jprcv_oemp/) )%laction = .TRUE.
     CASE( 'mixed oce-ice' )   ;   srcv( (/jprcv_rain, jprcv_snow,             jprcv_tevp/) )%laction = .TRUE. 
     CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_emp' )
     END SELECT


     !-------------------------------------
     ! wind stress : utau, vtau, utaui_ice, vtaui_ice 
     ! oce stress
     nrcv = nrcv + 1   ;   jprcv_otx1 = nrcv   ;   srcv(nrcv)%clname = 'O_OTaux1' ! oce tau 1st component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_oty1 = nrcv   ;   srcv(nrcv)%clname = 'O_OTauy1' ! oce tau 2nd component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_otz1 = nrcv   ;   srcv(nrcv)%clname = 'O_OTauz1' ! oce tau 3rd component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_otx2 = nrcv   ;   srcv(nrcv)%clname = 'O_OTaux2' ! oce tau 1st component on 2nd grid
     nrcv = nrcv + 1   ;   jprcv_oty2 = nrcv   ;   srcv(nrcv)%clname = 'O_OTauy2' ! oce tau 2nd component on 2nd grid
     nrcv = nrcv + 1   ;   jprcv_otz2 = nrcv   ;   srcv(nrcv)%clname = 'O_OTauz2' ! oce tau 3rd component on 2nd grid
     ! ice stress
     nrcv = nrcv + 1   ;   jprcv_itx1 = nrcv   ;   srcv(nrcv)%clname = 'O_ITaux1' ! ice tau 1st component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_ity1 = nrcv   ;   srcv(nrcv)%clname = 'O_ITauy1' ! ice tau 2nd component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_itz1 = nrcv   ;   srcv(nrcv)%clname = 'O_ITauz1' ! ice tau 3rd component on 1st grid
     nrcv = nrcv + 1   ;   jprcv_itx2 = nrcv   ;   srcv(nrcv)%clname = 'O_ITaux2' ! ice tau 1st component on 2nd grid
     nrcv = nrcv + 1   ;   jprcv_ity2 = nrcv   ;   srcv(nrcv)%clname = 'O_ITauy2' ! ice tau 2nd component on 2nd grid
     nrcv = nrcv + 1   ;   jprcv_itz2 = nrcv   ;   srcv(nrcv)%clname = 'O_ITauz2' ! ice tau 3rd component on 2nd grid
     ! change default definition of srcv(:)%nsgn
     srcv(jprcv_otx1:jprcv_itz2)%nsgn = -1
     ! change default definition of srcv(:)%clgrid and srcv(:)%laction
     
     DO i=1,5 
       cltmp(i)= cn_rcv_stress_4(i:i) 
     ENDDO
     
     SELECT CASE (LEN_TRIM(cn_rcv_stress(4)))   !  'T' 'U,V' 'U,V,F' 'U,V,I' 'T,F' 'T,I' 'T,U,V'
     CASE( 1 )   ! 'T'
         srcv(jprcv_otx1:jprcv_itz2)%clgrid = cltmp(1)   ! all oce and ice components on the same unique grid
         srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.   ! oce components on 1 grid 
         srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.   ! ice components on 1 grid 
     CASE( 3 )   ! 'U,V' 'T,F' 'T,I'
         SELECT CASE (cltmp(1))
         CASE( 'T' )   ! 'T,F' 'T,I'
             srcv(jprcv_otx1:jprcv_otz2)%clgrid = cltmp(1)   ! oce and ice tau on 2 grids
             srcv(jprcv_itx1:jprcv_itz2)%clgrid = cltmp(3)   ! but oce(ice) components on the same grid
             srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.   ! oce components on 1 grid
             srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.   ! ice components on 1 grid
         CASE( 'U' )   ! 'U,V'
             IF ( cltmp(3) /= 'V' ) CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
             srcv(jprcv_otx1:jprcv_otz1)%clgrid = cltmp(1)   ! oce(ice) components on 2 grids
             srcv(jprcv_otx2:jprcv_otz2)%clgrid = cltmp(3)
             srcv(jprcv_itx1:jprcv_itz1)%clgrid = cltmp(1)
             srcv(jprcv_itx2:jprcv_itz2)%clgrid = cltmp(3)
             srcv(jprcv_otx1:jprcv_itz2)%laction = .TRUE.   ! oce(ice) components on 2 grids
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
         END SELECT
     CASE( 5 )   ! 'U,V,F' 'U,V,I' 'T,U,V'
         SELECT CASE (cltmp(1))
         CASE( 'T' )   ! 'T,U,V'
             srcv(jprcv_otx1:jprcv_otz2)%clgrid = cltmp(1)    ! oce components on 1 grid
             srcv(jprcv_itx1:jprcv_itz1)%clgrid = cltmp(3)    ! ice components on 2 grids
             srcv(jprcv_itx2:jprcv_itz2)%clgrid = cltmp(5)
             srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.    ! oce components on 1 grid
             srcv(jprcv_itx1:jprcv_itz2)%laction = .TRUE.    ! ice components on 2 grids
         CASE( 'U' )   ! 'U,V,F' 'U,V,I' 
             IF ( cltmp(3) /= 'V' ) CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
             srcv(jprcv_otx1:jprcv_otz1)%clgrid = cltmp(1)    ! oce components on 2 grids
             srcv(jprcv_otx2:jprcv_otz2)%clgrid = cltmp(3)
             srcv(jprcv_itx1:jprcv_itz2)%clgrid = cltmp(5)    ! ice components on 1 grid
             srcv(jprcv_otx1:jprcv_otz2)%laction = .TRUE.    ! oce components on 2 grids 
             srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.    ! ice components on 1 grid 
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
         END SELECT
     CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
     END SELECT
     ! force .FALSE. to 3rd component for spherical coodinates
     IF ( TRIM(cn_rcv_stress(2)) == 'spherical' ) srcv((/jprcv_otz1, jprcv_otz2, jprcv_itz1, jprcv_itz2/))%laction = .FALSE. 
     ! force .FALSE. to ice components if not 'oce and ice'
     IF ( TRIM(cn_rcv_stress(1)) /= 'oce and ice' )   srcv(jprcv_itx1:jprcv_itz2)%laction = .FALSE.
     
     !-------------------------------------
     ! 10 m wind speed
     nrcv = nrcv + 1   ;   jprcv_w10m = nrcv   ;   srcv(nrcv)%clname = 'O_Wind10'
     IF ( TRIM(cn_rcv_w10m) == 'coupled' ) srcv(jprcv_w10m)%laction = .TRUE.
!     ! +++ ---> A brancher et a blinder dans tke  si TRIM(cn_rcv_w10m) == 'none'
     
     !-------------------------------------
     ! d(Qns)/d(T) 
     nrcv = nrcv + 1   ;   jprcv_dqnsdt = nrcv   ;   srcv(nrcv)%clname = 'O_dQnsdT'
     IF ( TRIM(cn_rcv_dqnsdt) == 'coupled' ) srcv(jprcv_dqnsdt)%laction = .TRUE.
     
     !-------------------------------------
     ! Runoff
     nrcv = nrcv + 1   ;   jprcv_rnf = nrcv   ;   srcv(nrcv)%clname = 'O_Runoff'
     IF ( TRIM(cn_rcv_rnf) /= 'climato' ) srcv(jprcv_rnf)%laction = .TRUE.
     
     !-------------------------------------
     ! Calving
     nrcv = nrcv + 1   ;   jprcv_cal = nrcv   ;   srcv(nrcv)%clname = 'OCalving'
     IF ( TRIM(cn_rcv_cal) == 'coupled' ) srcv(jprcv_cal)%laction = .TRUE.
     ! +++ ---> A brancher  
     
     !-------------------------------------
     ! fraction of net shortwave radiation which is not absorbed in the 
     ! thin surface layer and penetrates inside the ice cover 
     !  ( Maykut and Untersteiner, 1971 ; Elbert anbd Curry, 1993 )
     ! Since cloud cover catm not transmitted from atmosphere 
     ! ===> defined as constant value -> definition done in sbc_cpl_init
!!$      catm(:,:) = 0.
!!$      zcatm1(:,:) = 1.0 - catm(:,:)  !  fractional cloud cover
!!$      fr1_i0(:,:) = 0.18 * zcatm1(:,:) + 0.35 * catm(:,:) 
!!$      fr2_i0(:,:) = 0.82 * zcatm1(:,:) + 0.65 * catm(:,:)
     fr1_i0(:,:) = 0.18
     fr2_i0(:,:) = 0.82
     
     !
     !-------------------------------------
     !-------------------------------------
     ! Define the send interface
     !-------------------------------------
     !-------------------------------------
     !
     ! default definitions of nsnd
     nsnd = 0
     ssnd(:)%laction = .FALSE.
     ssnd(:)%clgrid = 'T'
     ssnd(:)%nsgn = 1
         
     !-------------------------------------
     ! Ice fraction
     nsnd = nsnd + 1   ;   jpsnd_fice = nsnd   ;   ssnd(nsnd)%clname = 'OIceFrac'
     ssnd(jpsnd_fice)%laction = .TRUE.
     
     !-------------------------------------
     ! T surf
     nsnd = nsnd + 1   ;   jpsnd_toce = nsnd   ;   ssnd(nsnd)%clname = 'O_SSTSST'
     nsnd = nsnd + 1   ;   jpsnd_tice = nsnd   ;   ssnd(nsnd)%clname = 'O_TepIce'
     nsnd = nsnd + 1   ;   jpsnd_tmix = nsnd   ;   ssnd(nsnd)%clname = 'O_TepMix'
     SELECT CASE (TRIM(cn_snd_temperature))
     CASE( 'oce only'             )   ;   ssnd(   jpsnd_toce               )%laction = .TRUE.
     CASE( 'weighted oce and ice' )   ;   ssnd( (/jpsnd_toce, jpsnd_tice/) )%laction = .TRUE.
     CASE( 'mixed oce-ice'        )   ;   ssnd(   jpsnd_tmix               )%laction = .TRUE.
     END SELECT

     !-------------------------------------
     ! Albedo
     nsnd = nsnd + 1   ;  jpsnd_albice = nsnd  ; ssnd(nsnd)%clname = 'O_AlbIce' 
     nsnd = nsnd + 1   ;  jpsnd_albmix = nsnd  ; ssnd(nsnd)%clname = 'O_AlbMix'
     SELECT CASE (TRIM(cn_snd_albedo))
     CASE( 'none'          )       ! nothing to do
     CASE( 'weighted ice'  )   ;   ssnd(jpsnd_albice)%laction = .TRUE.
     CASE( 'mixed oce-ice' )   ;   ssnd(jpsnd_albmix)%laction = .TRUE.
            CALL albedo_oce (albedo_oce_ov, albedo_oce_cs)
     ! Due to lack of information on nebulosity 
     ! albedo = ( albedo clear sky + albedo overcast sky ) / 2
            cpl_ocean_albedo = ( albedo_oce_cs + albedo_oce_ov ) / 2.
     END SELECT
         
     !-------------------------------------
     ! Thickness
     nsnd = nsnd + 1   ;   jpsnd_tckice = nsnd   ;   ssnd(nsnd)%clname = 'O_IceTck'
     nsnd = nsnd + 1   ;   jpsnd_tcksnw = nsnd   ;   ssnd(nsnd)%clname = 'O_SnwTck'
     IF ( TRIM(cn_snd_thickness) == 'weighted ice and snow' ) ssnd( (/jpsnd_tckice, jpsnd_tcksnw/) )%laction = .TRUE.
         
     !-------------------------------------
     ! Surface current
     nsnd = nsnd + 1   ;   jpsnd_ocx1 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCurx1'
     nsnd = nsnd + 1   ;   jpsnd_ocy1 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCury1'
     nsnd = nsnd + 1   ;   jpsnd_ocz1 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCurz1'
     nsnd = nsnd + 1   ;   jpsnd_ocx2 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCurx2'
     nsnd = nsnd + 1   ;   jpsnd_ocy2 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCury2'
     nsnd = nsnd + 1   ;   jpsnd_ocz2 = nsnd   ;   ssnd(nsnd)%clname = 'O_OCurz2'
     ! Ice velocity
     nsnd = nsnd + 1   ;   jpsnd_ivx1 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVelx1'
     nsnd = nsnd + 1   ;   jpsnd_ivy1 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVely1'
     nsnd = nsnd + 1   ;   jpsnd_ivz1 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVelz1'
     nsnd = nsnd + 1   ;   jpsnd_ivx2 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVelx2'
     nsnd = nsnd + 1   ;   jpsnd_ivy2 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVely2'
     nsnd = nsnd + 1   ;   jpsnd_ivz2 = nsnd   ;   ssnd(nsnd)%clname = 'O_IVelz2'

     ssnd(jpsnd_ocx1:jpsnd_ivz2)%nsgn = -1

     DO i = 1,3
        cltmp(i) = cn_snd_current_4(i:i)
     ENDDO

     SELECT CASE (LEN_TRIM(cn_snd_current(4)))   !  'T' 'U,V'
     CASE( 1 )   ! 'T'
         ssnd(jpsnd_ocx1:jpsnd_ivz2)%clgrid = cltmp(1)   ! all oce and ice components on the same unique grid
         ssnd(jpsnd_ocx1:jpsnd_ocz1)%laction = .TRUE.   ! oce components on 1 grid 
         ssnd(jpsnd_ivx1:jpsnd_ivz1)%laction = .TRUE.   ! ice components on 1 grid 
     CASE( 3 )   ! 'U,V' 
         IF ( cltmp(3) /= 'V' ) CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_snd_current(4)' )
         ssnd(jpsnd_ocx1:jpsnd_ocz1)%clgrid = cltmp(1)   ! oce(ice) components on 2 grids
         ssnd(jpsnd_ocx2:jpsnd_ocz2)%clgrid = cltmp(3)
         ssnd(jpsnd_ivx1:jpsnd_ivz1)%clgrid = cltmp(1)
         ssnd(jpsnd_ivx2:jpsnd_ivz2)%clgrid = cltmp(3)
         ssnd(jpsnd_ocx1:jpsnd_ivz2)%laction = .TRUE.   ! oce(ice) components on 2 grids
     CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_snd_current(4)' )
     END SELECT

     ! force .FALSE. to 3rd component for spherical coodinates
     IF ( TRIM(cn_snd_current(2)) == 'spherical' ) ssnd((/jpsnd_ocz1, jpsnd_ocz2, jpsnd_ivz1, jpsnd_ivz2/))%laction = .FALSE. 

     SELECT CASE (TRIM(cn_snd_current(1)))
     CASE( 'none'                 )   ;   ssnd( jpsnd_ocx1:jpsnd_ivz2 )%laction = .FALSE.
     CASE( 'oce only'             )   ;   ssnd( jpsnd_ivx1:jpsnd_ivz2 )%laction = .FALSE.
     CASE( 'weighted oce and ice' )   ! nothing to do
     CASE( 'mixed oce-ice'        )   ;   ssnd( jpsnd_ivx1:jpsnd_ivz2 )%laction = .FALSE.
     END SELECT
     !
     !-------------------------------------
     !-------------------------------------
     CALL cpl_prism_define
     !-------------------------------------
     !-------------------------------------
     !

   END SUBROUTINE sbc_cpl_init

   SUBROUTINE sbc_cpl_rcv( kt )
     

     INTEGER :: kt, isec, info, ji, jj

     !-------------------------------------
     ! Temporary buffers for receiving

     REAL(wp), DIMENSION(jpi,jpj) :: ztmp
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: qns_mix, qns_tmp, qns_ice_tmp 
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: qsr_mix, qsr_tmp, qsr_ice_tmp
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: zsnow, zrain, ztevp, zievp, tprecip_tmp, sprecip_tmp, emp_tmp
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: zotx1, zoty1, zotz1, zotx2, zoty2, zotz2 
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: zitx1, zity1, zitz1, zitx2, zity2, zitz2 
     REAL(wp), DIMENSION(jpi,jpj) :: ztmpx1, ztmpx2, ztmpy1, ztmpy2
     REAL(wp), DIMENSION(jpi,jpj), SAVE :: wind10_tmp, dqns_ice_tmp, rnfcpl_tmp, ocalving_tmp
! Eric: question - ces tableaux tmp ne devraient-ils pas etre initialises a zero ?

     IF( kt == nit000 )   CALL sbc_cpl_init
     
     isec = ( kt - nit000 ) * NINT(rdttra(1))        ! date of exchanges

     !-------------------------------------
     ! Qsr : we must get qsr and qsr_ice
     IF ( srcv(jprcv_qsroce)%laction ) CALL cpl_prism_rcv( jprcv_qsroce, isec, qsr_tmp    , info ) 
     IF ( srcv(jprcv_qsrice)%laction ) CALL cpl_prism_rcv( jprcv_qsrice, isec, qsr_ice_tmp, info ) 
     IF ( srcv(jprcv_qsrmix)%laction ) CALL cpl_prism_rcv( jprcv_qsrmix, isec, qsr_mix, info )


!!! Peut etre utiliser 
!      IF ( srcv(jprcv_qsrmix)%laction ) qsr(:,:) = ( qsr_mix(:,:) - freeze(:,:) * qsr_ice(:,:) ) / (1. - freeze(:,:))
!!!      IF (( srcv(jprcv_qsroce)%laction ) .and. ( srcv(jprcv_qsrice)%laction )) then
!!!            ztmp(:,:) = qsr_mix(:,:) / (1. - ( cpl_ocean_albedo(:,:)*(1. - freeze(:,:)) + freeze(:,:)*alb_ice(:,:)))
!!!            qsr_ice(:,:) = ztmp(:,:) * (1. - alb_ice(:,:))
!!!            qsr    (:,:) = ztmp(:,:) * (1. - cpl_ocean_albedo(:,:))
!!!        endif

     SELECT CASE (TRIM(cn_rcv_qsr))
     CASE( 'conservative' )
         qsr(:,:) = ( qsr_mix(:,:) - freeze(:,:) * qsr_ice_tmp(:,:) ) / (1. - freeze(:,:))
         qsr_ice(:,:) = qsr_ice_tmp(:,:)
     CASE( 'oce and ice' )
         qsr(:,:) =  qsr_tmp(:,:)
         qsr_ice(:,:) =  qsr_ice_tmp(:,:)
     CASE( 'mixed oce-ice' )
         ztmp(:,:) = qsr_mix(:,:) / (1. - ( cpl_ocean_albedo(:,:)*(1. - freeze(:,:)) + freeze(:,:)*alb_ice(:,:)))
         qsr_ice(:,:) = ztmp(:,:) * (1. - alb_ice(:,:))
         qsr    (:,:) = ztmp(:,:) * (1. - cpl_ocean_albedo(:,:))
     END SELECT

     !-------------------------------------
     ! d(Qns)/d(T)
     ! Must be read before Qns for mixed oce-ice case
     IF ( srcv(jprcv_dqnsdt)%laction )   CALL cpl_prism_rcv( jprcv_dqnsdt, isec, dqns_ice_tmp, info )
     dqns_ice(:,:) = dqns_ice_tmp(:,:)

     !-------------------------------------
     ! Qns : we must get qns and qns_ice
     IF ( srcv(jprcv_qnsoce)%laction ) CALL cpl_prism_rcv( jprcv_qnsoce, isec, qns_tmp    , info ) 
     IF ( srcv(jprcv_qnsice)%laction ) CALL cpl_prism_rcv( jprcv_qnsice, isec, qns_ice_tmp, info ) 
     IF ( srcv(jprcv_qnsmix)%laction ) CALL cpl_prism_rcv( jprcv_qnsmix, isec, qns_mix, info )        

     SELECT CASE (TRIM(cn_rcv_qns))
     CASE( 'conservative' )
         qns(:,:) = ( qns_mix(:,:) - freeze(:,:) * qns_ice_tmp(:,:) ) / (1. - freeze(:,:))
         qns_ice(:,:) = qns_ice_tmp(:,:)
     CASE( 'oce and ice' )
         qns(:,:) =  qns_tmp(:,:)
         qns_ice(:,:) =  qns_ice_tmp(:,:)
     CASE( 'mixed oce-ice' )

! Eric >> les valeurs freeze et tn_ice sont elles vraiment disponibles
!         a chaque pas de temps 
!         dans la version precedente de NEMO, elles l etaient uniquement
!         pour MOD( kt-1, nfice ) == 0

         ztmp(:,:) = tn_ice(:,:) * freeze(:,:) + ( tn(:,:,1) + rt0 ) * ( 1. - freeze(:,:) )
         qns(:,:) = qns_mix(:,:) + dqns_ice(:,:) * ( tn(:,:,1) + rt0 - ztmp(:,:) )
         qns_ice(:,:) = qns_mix(:,:) + dqns_ice(:,:) * ( tn_ice(:,:) - ztmp(:,:) )
     END SELECT

     !-------------------------------------
     ! Precipitations and Evaporation: we must get emp tprecip and sprecip
     ! sprecip = snow_ice - evap_ice
     ! tprecip = ( rain_ice + snow_ice ) - evap_ice
     ! emp     = emp_oce = evap_oce - ( rain_oce + snow_oce ) ... runoff??? ... calving???
     IF ( srcv(jprcv_snow)%laction )   CALL cpl_prism_rcv( jprcv_snow, isec, zsnow  , info ) ! snow
     IF ( srcv(jprcv_rain)%laction )   CALL cpl_prism_rcv( jprcv_rain, isec, zrain  , info ) ! Rain = liquid precipitation
     IF ( srcv(jprcv_tevp)%laction )   CALL cpl_prism_rcv( jprcv_tevp, isec, ztevp  , info ) ! total evaporation (over oce + ice)
     IF ( srcv(jprcv_ievp)%laction )   CALL cpl_prism_rcv( jprcv_ievp, isec, zievp  , info ) ! evaporation over ice (sublimation)
     IF ( srcv(jprcv_tpre)%laction )   CALL cpl_prism_rcv( jprcv_tpre, isec, tprecip_tmp, info ) ! see above
     IF ( srcv(jprcv_spre)%laction )   CALL cpl_prism_rcv( jprcv_spre, isec, sprecip_tmp, info ) ! see above
     IF ( srcv(jprcv_oemp)%laction )   CALL cpl_prism_rcv( jprcv_oemp, isec, emp_tmp    , info ) ! see above
     SELECT CASE (TRIM(cn_rcv_emp))
     CASE( 'conservative' )
         sprecip(:,:) = zsnow(:,:) - zievp(:,:)
         tprecip(:,:) = zrain(:,:) + sprecip(:,:)
!!! >> Arnaud pour seb et eric :  il me semble qu il y a une erreur  emp(:,:) = ( ztevp(:,:) - zievp(:,:)*(1. - freeze(:,:)) )/freeze(:,:) - tprecip(:,:)
         emp(:,:) = ( ztevp(:,:) - zievp(:,:)*freeze(:,:))/(1. - freeze(:,:)) - tprecip(:,:)
!!! << Arnaud
     CASE( 'oce and ice' )
         tprecip(:,:) = tprecip_tmp(:,:)
         sprecip(:,:) = sprecip_tmp(:,:)
         emp(:,:) = emp_tmp(:,:)
     CASE( 'mixed oce-ice' )
         tprecip(:,:) = zrain(:,:) + zsnow(:,:)
         sprecip(:,:) = zsnow(:,:)
         emp(:,:) = ztevp(:,:) - tprecip(:,:)
     END SELECT
     
     !-------------------------------------
     ! wind stress : we must get utau, vtau, utaui_ice, vtaui_ice 
     ! oce stress
     IF ( srcv(jprcv_otx1)%laction )   CALL cpl_prism_rcv( jprcv_otx1, isec, zotx1, info ) ! oce tau 1st component on 1st grid
     IF ( srcv(jprcv_oty1)%laction )   CALL cpl_prism_rcv( jprcv_oty1, isec, zoty1, info ) ! oce tau 2nd component on 1st grid
     IF ( srcv(jprcv_otz1)%laction )   CALL cpl_prism_rcv( jprcv_otz1, isec, zotz1, info ) ! oce tau 3rd component on 1st grid
     IF ( srcv(jprcv_otx2)%laction )   CALL cpl_prism_rcv( jprcv_otx2, isec, zotx2, info ) ! oce tau 1st component on 2nd grid
     IF ( srcv(jprcv_oty2)%laction )   CALL cpl_prism_rcv( jprcv_oty2, isec, zoty2, info ) ! oce tau 2nd component on 2nd grid
     IF ( srcv(jprcv_otz2)%laction )   CALL cpl_prism_rcv( jprcv_otz2, isec, zotz2, info ) ! oce tau 3rd component on 2nd grid
     ! ice stress
     IF ( srcv(jprcv_itx1)%laction )   CALL cpl_prism_rcv( jprcv_itx1, isec, zitx1, info ) ! ice tau 1st component on 1st grid
     IF ( srcv(jprcv_ity1)%laction )   CALL cpl_prism_rcv( jprcv_ity1, isec, zity1, info ) ! ice tau 2nd component on 1st grid
     IF ( srcv(jprcv_itz1)%laction )   CALL cpl_prism_rcv( jprcv_itz1, isec, zitz1, info ) ! ice tau 3rd component on 1st grid
     IF ( srcv(jprcv_itx2)%laction )   CALL cpl_prism_rcv( jprcv_itx2, isec, zitx2, info ) ! ice tau 1st component on 2nd grid
     IF ( srcv(jprcv_ity2)%laction )   CALL cpl_prism_rcv( jprcv_ity2, isec, zity2, info ) ! ice tau 2nd component on 2nd grid
     IF ( srcv(jprcv_itz2)%laction )   CALL cpl_prism_rcv( jprcv_itz2, isec, zitz2, info ) ! ice tau 3rd component on 2nd grid
     ! cartesian to spherical coordinates -> 3 components to 2 components
     IF ( TRIM(cn_rcv_stress(2)) == 'cartesian' ) THEN
         ! wind stress over ocean
         SELECT CASE (srcv(jprcv_otx1)%clgrid) 
         CASE( 'T' ) 
             CALL geo2oce ( zotx1, zoty1, zotz1, 't', glamt, gphit, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the same grid
         CASE( 'F' )
             CALL geo2oce ( zotx1, zoty1, zotz1, 'f', glamf, gphif, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the same grid
         CASE( 'U' )
             CALL geo2oce ( zotx1, zoty1, zotz1, 'u', glamu, gphiu, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the 1st grid
             CALL geo2oce ( zotx2, zoty2, zotz2, 'v', glamv, gphiv, ztmpx2, ztmpy2 ) ! 1st and 2nd components on the 2nd grid
             zotx2(:,:) = ztmpx2(:,:)   ! overwrite 1st component on the 2nd grid
             zoty2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
         END SELECT
         zotx1(:,:) = ztmpx1(:,:)   ! overwrite 1st component on the 1st grid
         zoty1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
         ! wind stress over ice
         IF ( srcv(jprcv_itx1)%laction ) THEN 
             SELECT CASE (srcv(jprcv_itx1)%clgrid) 
             CASE( 'T' ) 
                 CALL geo2oce ( zitx1, zity1, zitz1, 't', glamt, gphit, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the same grid
             CASE( 'F' )
                 CALL geo2oce ( zitx1, zity1, zitz1, 'f', glamf, gphif, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the same grid
             CASE( 'U' )
                 CALL geo2oce ( zitx1, zity1, zitz1, 'u', glamu, gphiu, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the 1st grid
                 CALL geo2oce ( zitx2, zity2, zitz2, 'v', glamv, gphiv, ztmpx2, ztmpy2 ) ! 1st and 2nd comp. on the 2nd grid
                 zitx2(:,:) = ztmpx2(:,:)   ! overwrite 1st comp. on the 2nd grid
                 zity2(:,:) = ztmpy2(:,:)   ! overwrite 2nd comp. on the 2nd grid
             END SELECT
             zitx1(:,:) = ztmpx1(:,:)   ! overwrite 1st comp. on the 1st grid
             zity1(:,:) = ztmpy1(:,:)   ! overwrite 2nd comp. on the 2nd grid
         ENDIF
     ENDIF
     
     ! 'eastward-northward' to 'local grid' axes -> rotate the components
     IF ( TRIM(cn_rcv_stress(3)) == 'eastward-northward' ) THEN                        ! Oce component
         CALL rot_rep( zotx1, zoty1, srcv(jprcv_otx1)%clgrid, 'en->i', ztmpx1 )      ! 1st component on the 1st grid
         zotx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
         IF ( srcv(jprcv_otx2)%laction ) THEN 
             CALL rot_rep( zotx2, zoty2, srcv(jprcv_otx2)%clgrid, 'en->j', ztmpy2 )   ! 2nd component on the 2nd grid
             zoty2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
         ELSE
             CALL rot_rep( zotx1, zoty1, srcv(jprcv_otx1)%clgrid, 'en->j', ztmpy1 )   ! 2nd component on the 1st grid
             zoty1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
         ENDIF
         IF ( srcv(jprcv_itx1)%laction ) THEN                                            ! Ice component
             CALL rot_rep( zitx1, zity1, srcv(jprcv_itx1)%clgrid, 'en->i', ztmpx1 )      ! 1st component on the 1st grid
             zitx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
             IF ( srcv(jprcv_itx2)%laction ) THEN
                 CALL rot_rep( zitx2, zity2, srcv(jprcv_itx2)%clgrid, 'en->j', ztmpy2 )   ! 2nd component on the 2nd grid
                 zity2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
             ELSE
                 CALL rot_rep( zitx1, zity1, srcv(jprcv_itx1)%clgrid, 'en->j', ztmpy1 )   ! 2nd component on the 1st grid
                 zity1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
             ENDIF
         ENDIF
     ENDIF
     
     ! oce stress must be on U,V grids
     IF ( srcv(jprcv_otx1)%clgrid == 'T' ) THEN
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             utau(ji,jj) = 0.5 * ( zotx1(ji,jj) + zotx1(ji+1,jj  ) ) ! T -> U grid
             vtau(ji,jj) = 0.5 * ( zoty1(ji,jj) + zoty1(ji  ,jj+1) ) ! T -> V grid
           END DO
         END DO
         CALL lbc_lnk( utau, 'U',  -1. )   ;   CALL lbc_lnk( vtau, 'V',  -1. )
     ELSE
         utau(:,:) = zotx1(:,:)
         vtau(:,:) = zoty2(:,:)
     ENDIF
     
     ! make sure we have stress over ice
     IF ( TRIM(cn_rcv_stress(1)) /= 'oce and ice' ) THEN 
         zitx1(:,:) = zotx1(:,:)                                                ! 1st component on the 1st grid
         IF ( srcv(jprcv_otx2)%laction ) THEN   ;   zity2(:,:) = zoty2(:,:)   ! 2nd component on the 2nd grid
         ELSE                                     ;   zity1(:,:) = zoty1(:,:)   ! 2nd component on the 1st grid
         ENDIF
         srcv(jprcv_itx1)%clgrid = srcv(jprcv_otx1)%clgrid   ! update grid of the ice component
     ENDIF
     
     ! ice stress must be on I grid
     SELECT CASE ( srcv(jprcv_itx1)%clgrid )
     CASE( 'U' )
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             utaui_ice(ji,jj) = 0.5 * ( zitx1(ji-1,jj  ) + zitx1(ji-1,jj-1) ) ! U -> I grid
             vtaui_ice(ji,jj) = 0.5 * ( zity2(ji  ,jj-1) + zity2(ji-1,jj-1) ) ! V -> I grid
           END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
     CASE( 'F' )
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             utaui_ice(ji,jj) = zitx1(ji-1,jj-1) ! F -> I grid
             vtaui_ice(ji,jj) = zity1(ji-1,jj-1) ! F -> I grid
           END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
     CASE( 'T' )
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             utaui_ice(ji,jj) = 0.25 * ( zitx1(ji,jj) + zitx1(ji-1,jj) + zitx1(ji,jj-1) + zitx1(ji-1,jj-1) ) ! T -> I grid
             vtaui_ice(ji,jj) = 0.25 * ( zity1(ji,jj) + zity1(ji-1,jj) + zity1(ji,jj-1) + zity1(ji-1,jj-1) ) ! T -> I grid
           END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
     CASE( 'I' )
         utaui_ice(:,:) = zitx1(:,:)
         vtaui_ice(:,:) = zity1(:,:)
     END SELECT
     
     
     !-------------------------------------
     ! 10 m wind speed
     ! +++ ---> blinder dans tke  si TRIM(cn_rcv_w10m) == 'none'
     IF ( srcv(jprcv_w10m  )%laction )   CALL cpl_prism_rcv( jprcv_w10m, isec, wind10_tmp, info )
!     wind10m(:,:) = wind10_tmp(:,:)
     
     !-------------------------------------
     ! Runoff
     ! C a u t i o n : runoff must be positive and in kg/m2/s 
     !
     IF ( srcv(jprcv_rnf   )%laction )   CALL cpl_prism_rcv( jprcv_rnf   , isec, rnfcpl_tmp, info )
     rnfcpl(:,:) = rnfcpl_tmp(:,:)
     !
     SELECT CASE (TRIM(cn_rcv_rnf))
     !
     CASE( 'climato' )
     !
     rnfcpl(:,:)=0.
     !
     CASE( 'coupled' )
     !
     ! remove negative runoff
     !
     mskpos(:,:)=0 ; mskneg(:,:)=0
     !
     WHERE ( rnfcpl(:,:) < 0. ) mskneg(:,:)=1 ; WHERE ( rnfcpl(:,:) > 0. ) mskpos(:,:)=1
     !
     zcumulpos=SUM(rnfcpl(:,:)*e1t(:,:)*e2t(:,:)*mskpos(:,:)) 
     zcumulneg=SUM(rnfcpl(:,:)*e1t(:,:)*e2t(:,:)*mskneg(:,:))
     !
! Eric: pas bien sur de ce mpp_sum la ... Seb ?
     IF( lk_mpp )   CALL mpp_sum( zcumulpos ) ! sum over the global domain
     IF( lk_mpp )   CALL mpp_sum( zcumulneg ) 
     !
     IF ( zcumulpos /= 0. ) THEN ; zcumulneg = zcumulneg / zcumulpos
     ELSE ; zcumulneg = 0
     ENDIF
     !
     ! distribute negative runoff on positive runoff grid points
     !
     DO jj = 2, jpjm1
        DO ji = fs_2, fs_jpim1   ! vector opt.
           rnfcpl(ji,jj) = rnfcpl(ji,jj) * ( 1. + zcumulneg ) * mskpos(ji,jj)
        ENDDO
     ENDDO
     !
     ! add runoff to e-p 
     !
     emp (:,:) = emp (:,:) - rnfcpl(:,:)
     emps(:,:) = emps(:,:) - rnfcpl(:,:)
     !
     CASE( 'mixed' )
     !
     ! sum to e-p to be done in sbc_rnf
     !
     END SELECT
     !
     !-------------------------------------
     ! Calving
     IF ( srcv(jprcv_cal   )%laction )   CALL cpl_prism_rcv( jprcv_cal   , isec, ocalving_tmp, info )
     ocalving(:,:) = ocalving_tmp(:,:)
     emp (:,:) = emp (:,:) - ABS (ocalving(:,:))
     emps(:,:) = emps(:,:) - ABS (ocalving(:,:))
     
     !  fraction of net shortwave radiation which is not absorbed in the 
     !  thin surface layer and penetrates inside the ice cover 
     !  ( Maykut and Untersteiner, 1971 ; Elbert anbd Curry, 1993 )
     !------------------------------------------------------------------
     ! Since cloud cover catm not transmitted from atmosphere
     ! ===> defined as constant value -> definition done in sbc_cpl_init
     
   END SUBROUTINE sbc_cpl_rcv
   
   SUBROUTINE sbc_cpl_snd( kt )
     
     
     INTEGER :: kt, isec, info, ji, jj
     REAL(wp), DIMENSION(jpi,jpj) :: ztmp            
     REAL(wp), DIMENSION(jpi,jpj) :: zotx1, zoty1, zotz1, zitx1, zity1, zitz1
     REAL(wp), DIMENSION(jpi,jpj) :: zotx2, zoty2, zotz2, zitx2, zity2, zitz2
     REAL(wp), DIMENSION(jpi,jpj) :: ztmpx1, ztmpy1, ztmpx2, ztmpy2
     
     isec = ( kt - nit000 ) * NINT(rdttra(1))        ! date of exchanges
     
     !-------------------------------------
     ! Ice fraction
     IF ( ssnd(jpsnd_fice)%laction ) CALL cpl_prism_snd( jpsnd_fice, isec, freeze, info )

     !-------------------------------------
     ! T surf
     ztmp(:,:) = tn(:,:,1) + rt0
     SELECT CASE (TRIM(cn_snd_temperature))
     CASE( 'oce only'             )       ! nothing to do
     CASE( 'weighted oce and ice' )   ;   ztmp = ztmp(:,:) * (1. - freeze(:,:))
     CASE( 'mixed oce-ice'        )   ;   ztmp = ztmp(:,:) * (1. - freeze(:,:)) + tn_ice(:,:)*freeze(:,:)
     END SELECT
     IF ( ssnd(jpsnd_toce)%laction ) CALL cpl_prism_snd( jpsnd_toce, isec, ztmp, info )
     IF ( ssnd(jpsnd_tice)%laction ) CALL cpl_prism_snd( jpsnd_tice, isec, tn_ice(:,:) * freeze(:,:), info )
     IF ( ssnd(jpsnd_tmix)%laction ) CALL cpl_prism_snd( jpsnd_tmix, isec, ztmp, info )
     
     !-------------------------------------
     ! Albedo
     IF ( ssnd(jpsnd_albice)%laction ) CALL cpl_prism_snd( jpsnd_albice, isec, alb_ice(:,:) * freeze(:,:), info )
     IF ( ssnd(jpsnd_albmix)%laction ) THEN 
         CALL cpl_prism_snd( jpsnd_albmix, isec, cpl_ocean_albedo * (1. - freeze(:,:)) + alb_ice(:,:) * freeze(:,:), info )
     ENDIF
      
     !-------------------------------------
     ! Thickness
     IF ( ssnd(jpsnd_tckice)%laction ) CALL cpl_prism_snd( jpsnd_tckice, isec, tckice(:,:) * freeze(:,:), info )
     IF ( ssnd(jpsnd_tcksnw)%laction ) CALL cpl_prism_snd( jpsnd_tcksnw, isec, tcksnw(:,:) * freeze(:,:), info )
      
     !-------------------------------------
     ! Surface current
     
     !-------------------------------------
     ! oce currents

     zotx1(:,:) = un(:,:,1)
     zoty1(:,:) = vn(:,:,1)
     zitx1 = utaui_ice
     zity1 = vtaui_ice
     
     SELECT CASE (TRIM(cn_snd_current_1))
     CASE( 'oce only'             )   ! nothing to do
     CASE( 'weighted oce and ice' )   ;  zotx1(:,:) = zotx1(:,:) * (1. - freeze(:,:)) ;  zoty1(:,:) = zoty1(:,:) * freeze(:,:)
                                         zitx1(:,:) = zitx1(:,:) * (1. - freeze(:,:)) ;  zity1(:,:) = zity1(:,:) * freeze(:,:)
     CASE( 'mixed oce-ice'        )   ;  zotx1(:,:) = zotx1(:,:) * (1. - freeze(:,:)) + zitx1(:,:)*freeze(:,:) ; zoty1 = zoty1(:,:) * (1. - freeze(:,:)) + zity1(:,:)*freeze(:,:)
     END SELECT

     zotx2(:,:) = zotx1(:,:)
     zoty2(:,:) = zoty1(:,:)
     zitx2(:,:) = zitx1(:,:)
     zity2(:,:) = zity1(:,:)

     IF ( ssnd(jpsnd_ocx1)%clgrid == 'T' ) THEN
         ! Interpolate current on from U,V grid
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             zotx2(ji,jj) = 0.5 * ( zotx1(ji-1,jj  ) + zotx1(ji,jj) ) ! U -> T grid
             zoty2(ji,jj) = 0.5 * ( zoty1(ji  ,jj-1) + zoty1(ji,jj) ) ! V -> T grid
           END DO
         END DO
! Eric:  1 ou -1 pour le troisieme argument de lbc_lnk ?
         CALL lbc_lnk( zotx2, 'T',  1. )   ;   CALL lbc_lnk( zoty2, 'T',  1. )
         zotx1(:,:) = zotx2(:,:)
         zoty1(:,:) = zoty2(:,:)
         !
         IF ( ssnd(jpsnd_ivx1)%laction ) THEN
         ! Interpolate ice velocities from I grid
           DO jj = 2, jpjm1
             DO ji = fs_2, fs_jpim1   ! vector opt.
               zitx2(ji,jj) = 0.25 * ( zitx1(ji-1,jj  ) + zitx1(ji  ,jj  ) + zitx1(ji-1,jj-1) + zitx1(ji  ,jj-1) ) ! I -> T grid
               zity2(ji,jj) = 0.25 * ( zity1(ji-1,jj  ) + zity1(ji  ,jj  ) + zity1(ji-1,jj-1) + zity1(ji  ,jj-1) ) ! I -> T grid
             END DO
           END DO
           CALL lbc_lnk( zitx2, 'T',  1. )   ;   CALL lbc_lnk( zity2, 'T',  1. )
           zitx1(:,:) = zitx2(:,:)
           zity1(:,:) = zity2(:,:)
           !
         ENDIF
     ELSE IF ( ssnd(jpsnd_ivx1)%laction ) THEN
         !
         ! Interpolate ice velocities from I grid
         DO jj = 2, jpjm1
           DO ji = fs_2, fs_jpim1   ! vector opt.
             ztmpx1(ji,jj) = 0.5 * ( zitx1(ji+1,jj) + zitx1(ji+1,jj+1) ) ! I -> U grid
             zitx2(ji,jj) = 0.5 * ( zitx1(ji,jj+1) + zitx1(ji+1,jj+1) ) ! I -> V grid
             zitx1(ji,jj) = ztmpx1(ji,jj)
             ztmpx1(ji,jj) = 0.5 * ( zity1(ji+1,jj) + zity1(ji+1,jj+1) ) ! I -> U grid
             zity2(ji,jj) = 0.5 * ( zity1(ji,jj+1) + zity1(ji+1,jj+1) ) ! I -> V grid
             zity1(ji,jj) = ztmpx1(ji,jj)
           END DO
         END DO
         CALL lbc_lnk( zitx1, 'U',  -1. )   ;   CALL lbc_lnk( zity1, 'U',  -1. )
         CALL lbc_lnk( zitx2, 'V',  -1. )   ;   CALL lbc_lnk( zity2, 'V',  -1. )
         !
     ENDIF

     ! 'local grid' to 'eastward-northward' axes -> rotate the components
     IF ( TRIM(cn_snd_current(3)) == 'eastward-northward' ) THEN                        ! Oce component
         !
         CALL rot_rep( zotx1, zoty1, ssnd(jpsnd_ocx1)%clgrid, 'ij->e', ztmpx1 )      ! 1st component on the 1st grid
         zotx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
         CALL rot_rep( zotx1, zoty1, srcv(jpsnd_ocx1)%clgrid, 'ij->n', ztmpy1 )   ! 2nd component on the 1st grid
         zoty1(:,:) = ztmpy1(:,:)   ! overwnrite 2nd component on the 1st grid
         !
         IF ( ssnd(jpsnd_ocx2)%laction ) THEN 
             CALL rot_rep( zotx2, zoty2, ssnd(jpsnd_ocx2)%clgrid, 'ij->e', ztmpx2 )   ! 1st component on the 2nd grid
             zotx2(:,:) = ztmpx2(:,:)   ! overwrite 2nd component on the 2nd grid
             CALL rot_rep( zotx2, zoty2, ssnd(jpsnd_ocy2)%clgrid, 'ij->n', ztmpy2 )   ! 2nd component on the 2nd grid
             zoty2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
         ENDIF
         !
         IF ( ssnd(jpsnd_ivx1)%laction ) THEN                                            ! Ice component
             CALL rot_rep( zitx1, zity1, ssnd(jpsnd_ivx1)%clgrid, 'ij->e', ztmpx1 )      ! 1st component on the 1st grid
             zitx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
             CALL rot_rep( zitx1, zity1, ssnd(jpsnd_ivx1)%clgrid, 'ij->n', ztmpy1 )   ! 2nd component on the 1st grid
             zity1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
             !
             IF ( ssnd(jpsnd_ivx2)%laction ) THEN
                 CALL rot_rep( zitx2, zity2, ssnd(jpsnd_ivx2)%clgrid, 'ij->e', ztmpx2 )   ! 1st component on the 2nd grid
                 zitx2(:,:) = ztmpx2(:,:)   ! overwrite 2nd component on the 2nd grid
                 CALL rot_rep( zitx2, zity2, ssnd(jpsnd_ivx2)%clgrid, 'ij->n', ztmpy2 )   ! 2nd component on the 2nd grid
                 zity2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
             ELSE
             ENDIF
         ENDIF
     ENDIF

! Eric : Arnaud, je te laisse coder oce2geo !     
     ! spherical coordinates to cartesian -> 2 components to 3 components
     IF ( TRIM(cn_snd_current(2)) == 'cartesian' ) THEN
         ! oceanic currents
         ztmpx1(:,:) = zotx1(:,:)
         ztmpy1(:,:) = zoty1(:,:)
         SELECT CASE (ssnd(jpsnd_ocx1)%clgrid) 
         CASE( 'T' ) 
             CALL oce2geo ( ztmpx1, ztmpy1, 't', glamt, gphit, zotx1, zoty1, zotz1 ) ! 1st and 2nd components on the same grid
         CASE( 'U' )
             CALL oce2geo ( ztmpx1, ztmpy1, 'u', glamu, gphiu, zotx1, zoty1, zotz1 ) ! 1st and 2nd components on the 1st grid
             ztmpx2(:,:) = zotx2(:,:)
             ztmpy2(:,:) = zoty2(:,:)
             CALL oce2geo ( ztmpx2, ztmpy2, 'v', glamv, gphiv, zotx2, zoty2, zotz2 ) ! 1st and 2nd components on the 2nd grid
         END SELECT
         !
         ! ice velocities
         IF ( ssnd(jpsnd_ivx1)%laction ) THEN 
             ztmpx1(:,:) = zitx1(:,:)
             ztmpy1(:,:) = zity1(:,:)
             SELECT CASE (ssnd(jpsnd_ivx1)%clgrid) 
             CASE( 'T' ) 
                 CALL oce2geo ( ztmpx1, ztmpy1, 't', glamt, gphit, zitx1, zity1, zitz1 ) ! 1st and 2nd components on the same grid
             CASE( 'U' )
                 CALL oce2geo ( ztmpx1, ztmpy1, 'u', glamu, gphiu, zitx1, zity1, zitz1 ) ! 1st and 2nd components on the 1st grid
                 ztmpx2(:,:) = zitx2(:,:)
                 ztmpy2(:,:) = zity2(:,:)
                 CALL oce2geo ( ztmpx2, ztmpy2, 'v', glamv, gphiv, zitx2, zity2, zitz2 ) ! 1st and 2nd components on the 2nd grid
             END SELECT
         ENDIF
     ENDIF

     IF ( ssnd(jpsnd_ocx1)%laction )   CALL cpl_prism_snd( jpsnd_ocx1, isec, zotx1, info ) ! oce x current first grid
     IF ( ssnd(jpsnd_ocy1)%laction )   CALL cpl_prism_snd( jpsnd_ocy1, isec, zoty1, info ) ! oce y current first grid
     IF ( ssnd(jpsnd_ocz1)%laction )   CALL cpl_prism_snd( jpsnd_ocz1, isec, zotz1, info ) ! oce z current first grid
     IF ( ssnd(jpsnd_ocx2)%laction )   CALL cpl_prism_snd( jpsnd_ocx2, isec, zotx2, info ) ! oce x current second grid
     IF ( ssnd(jpsnd_ocy2)%laction )   CALL cpl_prism_snd( jpsnd_ocy2, isec, zoty2, info ) ! oce y current second grid
     IF ( ssnd(jpsnd_ocz2)%laction )   CALL cpl_prism_snd( jpsnd_ocz2, isec, zotz2, info ) ! oce z current second grid

     IF ( ssnd(jpsnd_ivx1)%laction )   CALL cpl_prism_snd( jpsnd_ivx1, isec, zitx1, info ) ! ice x current first grid
     IF ( ssnd(jpsnd_ivy1)%laction )   CALL cpl_prism_snd( jpsnd_ivy1, isec, zity1, info ) ! ice y current first grid
     IF ( ssnd(jpsnd_ivz1)%laction )   CALL cpl_prism_snd( jpsnd_ivz1, isec, zitz1, info ) ! ice z current first grid
     IF ( ssnd(jpsnd_ivx2)%laction )   CALL cpl_prism_snd( jpsnd_ivx2, isec, zitx2, info ) ! ice x current second grid
     IF ( ssnd(jpsnd_ivy2)%laction )   CALL cpl_prism_snd( jpsnd_ivy2, isec, zity2, info ) ! ice y current second grid
     IF ( ssnd(jpsnd_ivz2)%laction )   CALL cpl_prism_snd( jpsnd_ivz2, isec, zitz2, info ) ! ice z current second grid

   END SUBROUTINE sbc_cpl_snd
   
#else
      !!----------------------------------------------------------------------
      !!   Dummy routine                              NO coupling
      !!----------------------------------------------------------------------

+++ a verifier ...


#endif

      !!======================================================================
END MODULE sbccpl
